package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class QualifiedSuperReference extends QualifiedThisReference {
  public QualifiedSuperReference(TypeReference name, int pos, int sourceEnd) {
    super(name, pos, sourceEnd);
  }
  
  public boolean isSuper() {
    return true;
  }
  
  public boolean isQualifiedSuper() {
    return true;
  }
  
  public boolean isThis() {
    return false;
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    return this.qualification.print(0, output).append(".super");
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    if ((this.bits & 0x1FE00000) != 0) {
      scope.problemReporter().invalidParenthesizedExpression((ASTNode)this);
      return null;
    } 
    super.resolveType(scope);
    if (this.resolvedType != null && !this.resolvedType.isValidBinding()) {
      scope.problemReporter().illegalSuperAccess(this.qualification.resolvedType, this.resolvedType, (ASTNode)this);
      return null;
    } 
    if (this.currentCompatibleType == null)
      return null; 
    if (this.currentCompatibleType.id == 1) {
      scope.problemReporter().cannotUseSuperInJavaLangObject((ASTNode)this);
      return null;
    } 
    return this.resolvedType = this.currentCompatibleType.isInterface() ? 
      (TypeBinding)this.currentCompatibleType : 
      (TypeBinding)this.currentCompatibleType.superclass();
  }
  
  int findCompatibleEnclosing(ReferenceBinding enclosingType, TypeBinding type, BlockScope scope) {
    if (type.isInterface()) {
      CompilerOptions compilerOptions = scope.compilerOptions();
      ReferenceBinding[] supers = enclosingType.superInterfaces();
      int length = supers.length;
      boolean isJava8 = (compilerOptions.complianceLevel >= 3407872L);
      boolean isLegal = true;
      char[][] compoundName = null;
      ReferenceBinding closestMatch = null;
      for (int i = 0; i < length; i++) {
        if (TypeBinding.equalsEquals(supers[i].erasure(), type)) {
          this.currentCompatibleType = closestMatch = supers[i];
        } else if (supers[i].erasure().isCompatibleWith(type)) {
          isLegal = false;
          compoundName = (supers[i]).compoundName;
          if (closestMatch == null)
            closestMatch = supers[i]; 
        } 
      } 
      if (!isLegal || !isJava8) {
        this.currentCompatibleType = null;
        this.resolvedType = (TypeBinding)new ProblemReferenceBinding(compoundName, 
            closestMatch, isJava8 ? 21 : 29);
      } 
      return 0;
    } 
    return super.findCompatibleEnclosing(enclosingType, type, scope);
  }
  
  public void traverse(ASTVisitor visitor, BlockScope blockScope) {
    if (visitor.visit(this, blockScope))
      this.qualification.traverse(visitor, blockScope); 
    visitor.endVisit(this, blockScope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope blockScope) {
    if (visitor.visit(this, blockScope))
      this.qualification.traverse(visitor, blockScope); 
    visitor.endVisit(this, blockScope);
  }
}
